@@ -16,7 +16,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: input.c,v 1.16 2009/01/25 18:50:32 menno Exp $
+ * $Id: input.c,v 1.21 2017/07/02 14:35:23 knik Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -78,7 +78,7 @@ struct WAVEFORMATEX
   u_int16_t wBitsPerSample;
   u_int16_t cbSize;
 }
-#ifdef __GNUC
+#ifdef __GNUC__
 __attribute__((packed))
 #endif
 ;
@@ -94,7 +94,7 @@ struct WAVEFORMATEXTENSIBLE
   u_int32_t dwChannelMask;		// which channels are present in stream
   unsigned char SubFormat[16];		// guid
 }
-#ifdef __GNUC
+#ifdef __GNUC__
 __attribute__((packed))
 #endif
 ;
@@ -116,17 +116,44 @@ static void unsuperr(const char *name)
   fprintf(stderr, "%s: file format not supported\n", name);
 }
 
+static void seekcur(FILE *f, int ofs)
+{
+    int cnt;
+
+    for (cnt = 0; cnt < ofs; cnt++)
+        fgetc(f);
+}
+
+static int seekchunk(FILE *f, riffsub_t *riffsub, char *name)
+{
+ int skipped;
+
+ for(skipped = 0; skipped < 10; skipped++)
+ {
+   if (fread(riffsub, 1, sizeof(*riffsub), f) != sizeof(*riffsub))
+     return 0;
+
+   riffsub->len = UINT32(riffsub->len);
+   if (riffsub->len & 1)
+     riffsub->len++;
+
+   if (!memcmp(&(riffsub->label), name, 4))
+     return 1;
+
+   seekcur(f, riffsub->len);
+ }
+
+ return 0;
+}
+
 pcmfile_t *wav_open_read(const char *name, int rawinput)
 {
-  int i;
-  int skip;
   FILE *wave_f;
   riff_t riff;
   riffsub_t riffsub;
   struct WAVEFORMATEXTENSIBLE wave;
   char *riffl = "RIFF";
   char *wavel = "WAVE";
-  char *bextl = "BEXT";
   char *fmtl = "fmt ";
   char *datal = "data";
   int fmtsize;
@@ -156,44 +183,26 @@ pcmfile_t *wav_open_read(const char *name, int rawinput)
     if (memcmp(&(riff.chunk_type), wavel, 4))
       return NULL;
 
-    // handle broadcast extensions. added by pro-tools,otherwise it must be fmt chunk.
-    if (fread(&riffsub, 1, sizeof(riffsub), wave_f) != sizeof(riffsub))
-        return NULL;
-    riffsub.len = UINT32(riffsub.len);
-
-    if (!memcmp(&(riffsub.label), bextl, 4))
-    {
-        fseek(wave_f, riffsub.len, SEEK_CUR);
-
-        if (fread(&riffsub, 1, sizeof(riffsub), wave_f) != sizeof(riffsub))
-            return NULL;
-        riffsub.len = UINT32(riffsub.len);
-    }
+    if (!seekchunk(wave_f, &riffsub, fmtl))
+      return NULL;
 
     if (memcmp(&(riffsub.label), fmtl, 4))
         return NULL;
     memset(&wave, 0, sizeof(wave));
 
     fmtsize = (riffsub.len < sizeof(wave)) ? riffsub.len : sizeof(wave);
-    if (fread(&wave, 1, fmtsize, wave_f) != fmtsize)
+    // check if format is at least 16 bytes long
+    if (fmtsize < 16)
+	return NULL;
+
+   if (fread(&wave, 1, fmtsize, wave_f) != fmtsize)
         return NULL;
 
-    for (skip = riffsub.len - fmtsize; skip > 0; skip--)
-      fgetc(wave_f);
+    seekcur(wave_f, riffsub.len - fmtsize);
 
-    for (i = 0;; i++)
-    {
-      if (fread(&riffsub, 1, sizeof(riffsub), wave_f) != sizeof(riffsub))
-	return NULL;
-      riffsub.len = UINT32(riffsub.len);
-      if (!memcmp(&(riffsub.label), datal, 4))
-	break;
-      if (i > 10)
-	return NULL;
+    if (!seekchunk(wave_f, &riffsub, datal))
+      return NULL;
 
-      for (skip = riffsub.len; skip > 0; skip--)
-	fgetc(wave_f);
-    }
     if (UINT16(wave.Format.wFormatTag) != WAVE_FORMAT_PCM && UINT16(wave.Format.wFormatTag) != WAVE_FORMAT_FLOAT)
     {
       if (UINT16(wave.Format.wFormatTag) == WAVE_FORMAT_EXTENSIBLE)
