@@ -23,9 +23,6 @@ copyright notice must be included in all copies or derivative works.
 
 Copyright (c) 1997.
 **********************************************************************/
-/*
- * $Id: bitstream.c,v 1.34 2007/06/05 18:59:47 menno Exp $
- */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,15 +31,14 @@ Copyright (c) 1997.
 #include "channels.h"
 #include "huffman.h"
 #include "bitstream.h"
-#include "ltp.h"
 #include "util.h"
 
-static int CountBitstream(faacEncHandle hEncoder,
+static int CountBitstream(faacEncStruct* hEncoder,
                           CoderInfo *coderInfo,
                           ChannelInfo *channelInfo,
                           BitStream *bitStream,
                           int numChannels);
-static int WriteADTSHeader(faacEncHandle hEncoder,
+static int WriteADTSHeader(faacEncStruct* hEncoder,
                            BitStream *bitStream,
                            int writeFlag);
 static int WriteCPE(CoderInfo *coderInfoL,
@@ -71,12 +67,6 @@ static int WriteICS(CoderInfo *coderInfo,
                     int commonWindow,
                     int objectType,
                     int writeFlag);
-static int WriteLTPPredictorData(CoderInfo *coderInfo,
-                                 BitStream *bitStream,
-                                 int writeFlag);
-static int WritePredictorData(CoderInfo *coderInfo,
-                              BitStream *bitStream,
-                              int writeFlag);
 static int WritePulseData(CoderInfo *coderInfo,
                           BitStream *bitStream,
                           int writeFlag);
@@ -152,7 +142,7 @@ static int WriteFAACStr(BitStream *bitStream, char *version, int write)
 }
 
 
-int WriteBitstream(faacEncHandle hEncoder,
+int WriteBitstream(faacEncStruct* hEncoder,
                    CoderInfo *coderInfo,
                    ChannelInfo *channelInfo,
                    BitStream *bitStream,
@@ -162,7 +152,8 @@ int WriteBitstream(faacEncHandle hEncoder,
     int bits = 0;
     int bitsLeftAfterFill, numFillBits;
 
-    CountBitstream(hEncoder, coderInfo, channelInfo, bitStream, numChannel);
+    if (CountBitstream(hEncoder, coderInfo, channelInfo, bitStream, numChannel) < 0)
+        return -1;
 
     if(hEncoder->config.outputFormat == 1){
         bits += WriteADTSHeader(hEncoder, bitStream, 1);
@@ -243,7 +234,7 @@ int WriteBitstream(faacEncHandle hEncoder,
     return bits;
 }
 
-static int CountBitstream(faacEncHandle hEncoder,
+static int CountBitstream(faacEncStruct* hEncoder,
                           CoderInfo *coderInfo,
                           ChannelInfo *channelInfo,
                           BitStream *bitStream,
@@ -326,10 +317,21 @@ static int CountBitstream(faacEncHandle hEncoder,
 
     hEncoder->usedBytes = bit2byte(bits);
 
+    if (hEncoder->usedBytes > bitStream->size)
+    {
+        fprintf(stderr, "frame buffer overrun\n");
+        return -1;
+    }
+    if (hEncoder->usedBytes >= ADTS_FRAMESIZE)
+    {
+        fprintf(stderr, "frame size limit exceeded\n");
+        return -1;
+    }
+
     return bits;
 }
 
-static int WriteADTSHeader(faacEncHandle hEncoder,
+static int WriteADTSHeader(faacEncStruct* hEncoder,
                            BitStream *bitStream,
                            int writeFlag)
 {
@@ -413,8 +415,8 @@ static int WriteCPE(CoderInfo *coderInfoL,
         int numWindows, maxSfb;
 
         bits += WriteICSInfo(coderInfoL, bitStream, objectType, channelInfo->common_window, writeFlag);
-        numWindows = coderInfoL->num_window_groups;
-        maxSfb = coderInfoL->max_sfb;
+        numWindows = coderInfoL->groups.n;
+        maxSfb = coderInfoL->sfbn;
 
         if (writeFlag) {
             PutBit(bitStream, channelInfo->msInfo.is_present, LEN_MASK_PRES);
@@ -519,7 +521,7 @@ static int WriteICSInfo(CoderInfo *coderInfo,
     /* For short windows, write out max_sfb and scale_factor_grouping */
     if (coderInfo->block_type == ONLY_SHORT_WINDOW){
         if (writeFlag) {
-            PutBit(bitStream, coderInfo->max_sfb, LEN_MAX_SFBS);
+            PutBit(bitStream, coderInfo->sfbn, LEN_MAX_SFBS);
             grouping_bits = FindGroupingBits(coderInfo);
             PutBit(bitStream, grouping_bits, MAX_SHORT_WINDOWS - 1);  /* the grouping bits */
         }
@@ -527,7 +529,7 @@ static int WriteICSInfo(CoderInfo *coderInfo,
         bits += MAX_SHORT_WINDOWS - 1;
     } else { /* Otherwise, write out max_sfb and predictor data */
         if (writeFlag) {
-            PutBit(bitStream, coderInfo->max_sfb, LEN_MAX_SFBL);
+            PutBit(bitStream, coderInfo->sfbn, LEN_MAX_SFBL);
         }
         bits += LEN_MAX_SFBL;
 #ifdef DRM
@@ -537,22 +539,9 @@ static int WriteICSInfo(CoderInfo *coderInfo,
     }
     bits += LEN_TNS_PRES;
 #endif
-        if (objectType == LTP)
-        {
-            bits++;
-            if(writeFlag)
-                PutBit(bitStream, coderInfo->ltpInfo.global_pred_flag, 1); /* Prediction Global used */
-
-            bits += WriteLTPPredictorData(coderInfo, bitStream, writeFlag);
-            if (common_window)
-                bits += WriteLTPPredictorData(coderInfo, bitStream, writeFlag);
-        } else {
             bits++;
             if (writeFlag)
-                PutBit(bitStream, coderInfo->pred_global_flag, LEN_PRED_PRES);  /* predictor_data_present */
-
-            bits += WritePredictorData(coderInfo, bitStream, writeFlag);
-        }
+                PutBit(bitStream, 0, LEN_PRED_PRES);  /* predictor_data_present */
 #ifndef DRM
     }
 #endif
@@ -622,86 +611,6 @@ static int WriteICS(CoderInfo *coderInfo,
     return bits;
 }
 
-static int WriteLTPPredictorData(CoderInfo *coderInfo, BitStream *bitStream, int writeFlag)
-{
-    int i, last_band;
-    int bits;
-    LtpInfo *ltpInfo = &coderInfo->ltpInfo;
-
-    bits = 0;
-
-    if (ltpInfo->global_pred_flag)
-    {
-
-        if(writeFlag)
-            PutBit(bitStream, 1, 1); /* LTP used */
-        bits++;
-
-        switch(coderInfo->block_type)
-        {
-        case ONLY_LONG_WINDOW:
-        case LONG_SHORT_WINDOW:
-        case SHORT_LONG_WINDOW:
-            bits += LEN_LTP_LAG;
-            bits += LEN_LTP_COEF;
-            if(writeFlag)
-            {
-                PutBit(bitStream, ltpInfo->delay[0], LEN_LTP_LAG);
-                PutBit(bitStream, ltpInfo->weight_idx,  LEN_LTP_COEF);
-            }
-
-            last_band = ((coderInfo->nr_of_sfb < MAX_LT_PRED_LONG_SFB) ?
-                coderInfo->nr_of_sfb : MAX_LT_PRED_LONG_SFB);
-//            last_band = coderInfo->nr_of_sfb;
-
-            bits += last_band;
-            if(writeFlag)
-                for (i = 0; i < last_band; i++)
-                    PutBit(bitStream, ltpInfo->sfb_prediction_used[i], LEN_LTP_LONG_USED);
-            break;
-
-        default:
-            break;
-        }
-    }
-
-    return (bits);
-}
-
-static int WritePredictorData(CoderInfo *coderInfo,
-                              BitStream *bitStream,
-                              int writeFlag)
-{
-    int bits = 0;
-
-    /* Write global predictor data present */
-    short predictorDataPresent = coderInfo->pred_global_flag;
-    int numBands = min(coderInfo->max_pred_sfb, coderInfo->nr_of_sfb);
-
-    if (writeFlag) {
-        if (predictorDataPresent) {
-            int b;
-            if (coderInfo->reset_group_number == -1) {
-                PutBit(bitStream, 0, LEN_PRED_RST); /* No prediction reset */
-            } else {
-                PutBit(bitStream, 1, LEN_PRED_RST);
-                PutBit(bitStream, (unsigned long)coderInfo->reset_group_number,
-                    LEN_PRED_RSTGRP);
-            }
-
-            for (b=0;b<numBands;b++) {
-                PutBit(bitStream, coderInfo->pred_sfb_flag[b], LEN_PRED_ENAB);
-            }
-        }
-    }
-    bits += (predictorDataPresent) ?
-        (LEN_PRED_RST +
-        ((coderInfo->reset_group_number)!=-1)*LEN_PRED_RSTGRP +
-        numBands*LEN_PRED_ENAB) : 0;
-
-    return bits;
-}
-
 static int WritePulseData(CoderInfo *coderInfo,
                           BitStream *bitStream,
                           int writeFlag)
@@ -913,8 +822,8 @@ static int FindGroupingBits(CoderInfo *coderInfo)
     int i, j;
     int index = 0;
 
-    for(i = 0; i < coderInfo->num_window_groups; i++){
-        for (j = 0; j < coderInfo->window_group_length[i]; j++){
+    for(i = 0; i < coderInfo->groups.n; i++){
+        for (j = 0; j < coderInfo->groups.len[i]; j++){
             tmp[index++] = i;
         }
     }
@@ -1092,7 +1001,7 @@ static int WriteReorderedSpectralData(CoderInfo *coderInfo,
     int segmcnt = 0;
     long startbitpos;
     segment_t segment[FRAME_LEN];
-    int* window_group_length = coderInfo->window_group_length;
+    int* groups = coderInfo->groups.len;
     int* sfb_offset = coderInfo->sfb_offset;
 
     cw_info_t cw_info[FRAME_LEN];
@@ -1137,7 +1046,7 @@ static int WriteReorderedSpectralData(CoderInfo *coderInfo,
         /* presort codewords ------------------------------------------------ */
         /* classify codewords first */
         sfb_cnt = win_cnt = win_grp_cnt = coeff_cnt = last_sfb = acc_win_cnt = 0;
-        cur_sfb_len = sfb_offset[1] / window_group_length[0];
+        cur_sfb_len = sfb_offset[1] / groups[0];
         cur_cb = coderInfo->book_vector[0];
         for (i = 0; i < num_cw; i++) {
             /* Set codeword info parameters */
@@ -1153,14 +1062,14 @@ static int WriteReorderedSpectralData(CoderInfo *coderInfo,
                 last_sfb += cur_sfb_len;
 
                 win_cnt++; /* next window */
-                if (win_cnt == window_group_length[win_grp_cnt]) {
+                if (win_cnt == groups[win_grp_cnt]) {
                     win_cnt = 0;
 
                     sfb_cnt++; /* next sfb */
                     if (sfb_cnt == coderInfo->all_sfb) {
                         sfb_cnt = 0;
 
-                        acc_win_cnt += window_group_length[win_grp_cnt];
+                        acc_win_cnt += groups[win_grp_cnt];
                         win_grp_cnt++; /* next window group */
                     }
 
@@ -1168,7 +1077,7 @@ static int WriteReorderedSpectralData(CoderInfo *coderInfo,
                     cur_cb = coderInfo->book_vector[sfb_cnt];
                     if (last_sfb < FRAME_LEN) {
                         cur_sfb_len = (sfb_offset[sfb_cnt + 1] - sfb_offset[sfb_cnt])
-                            / window_group_length[win_grp_cnt];
+                            / groups[win_grp_cnt];
                     }
                 }
             }
